using System;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Xml;
using System.Xml.XPath;

namespace innoQ.Ticket.Consumer {

    /// <summary>
    /// Ticket-Service-Interface
    /// </summary>
    [ServiceContract(Name = "TicketService", Namespace = "http://reg.innoQ.com/ns/wsdl/TicketService/2007-02")]
    internal interface ITicketService {
        [OperationContract(Name = "GetAllEvents")]
        Message GetAllEvents(Message request);

        [OperationContract(Name = "OrderTicket",
            Action = "http://reg.innoq.com/ns/wsdl/TicketService/2007-02/ITicketService/OrderTicket",
            ReplyAction = "http://reg.innoq.com/ns/wsdl/TicketService/2007-02/ITicketService/OrderTicketResponse")]
        Message OrderTicket(Message request);
    }

    /// <summary>
    /// TicketService Consumer
    /// </summary>
    internal class Consumer {
        /// <summary>
        /// Sende Message an Service ...
        /// </summary>
        /// <param name="eventId"></param>
        /// <param name="eventName"></param>
        /// <param name="location"></param>
        /// <param name="customer"></param>
        /// <param name="priceCategory"></param>
        /// <returns></returns>
        public Message SendOrderTicketMessage(Guid eventId, string eventName, string location, string customer,
                                              string priceCategory) {
            Message msg = null;
            using (ChannelFactory<ITicketService> fac = new ChannelFactory<ITicketService>("clientChannel")) {
                fac.Open();
                ITicketService channel = fac.CreateChannel();
                msg =
                    channel.OrderTicket(
                        ConsumerMessaging.CreateTicketOrder(eventId, eventName, location, customer, priceCategory));
                fac.Close();
            }
            return msg;
        }

        /// <summary>
        /// Main method.
        /// </summary>
        /// <param name="args"></param>
        private static void Main(string[] args) {
            Consumer consumer = new Consumer();
            Console.WriteLine("Press 'v' to send VIP-TicketOrder ...");
            Console.WriteLine("Press 's' to send Standard-TicketOrder ...");
            Console.WriteLine("Press 'n' to send Value-TicketOrder ...");
            Console.WriteLine("Press any other key to terminate ...");
            string priceCategory = "Value";
            while (priceCategory != null) {
                char pressedKey = Console.ReadKey(true).KeyChar;
                switch (pressedKey) {
                    case 'v':
                        priceCategory = "VIP";
                        break;
                    case 's':
                        priceCategory = "Standard";
                        break;
                    case 'n':
                        priceCategory = "Value";
                        break;
                    default:
                        priceCategory = null;
                        break;
                }
                if (priceCategory != null) {
                    Message msg = consumer.SendOrderTicketMessage(Guid.NewGuid(), "BASTA!", "Frankfurt", "Wilms", priceCategory);
                    if (!msg.IsFault) {
                        XPathNavigator nav = new XPathDocument(msg.GetReaderAtBodyContents()).CreateNavigator();
                        XmlNamespaceManager nmMan = new XmlNamespaceManager(nav.NameTable);
                        nmMan.AddNamespace("m", ConsumerMessaging.TICKET_MESSAGE_XSD_NAMESPACE);
                        nmMan.AddNamespace("t", ConsumerMessaging.TICKET_XSD_NAMESPACE);
                        Console.WriteLine("Ticket Id: {0}",
                                          nav.SelectSingleNode("/m:OrderTicketResponse/m:response/t:id", nmMan).Value);
                        Console.WriteLine("Ticket Price: {0}",
                                          nav.SelectSingleNode("/m:OrderTicketResponse/m:response/t:price", nmMan).Value);
                    } else {
                        Console.WriteLine("Fehler: {0}", msg.ToString());
                    }
                }
            }
        }
    }
}