using System;
using System.IO;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Text;
using System.Xml;
using System.Xml.Schema;

namespace innoQ.Ticket.Consumer {

    /// <summary>
    /// Hilfsmethoden fuer das XML-Messaging
    /// </summary>
    public static class ConsumerMessaging {
        public const string TICKET_XSD_NAMESPACE = "http://reg.innoQ.com/ns/xsd/TicketService/2007-02";
        public const string TICKET_MESSAGE_XSD_NAMESPACE = "http://reg.innoQ.com/ns/xsd/TicketService/Messages/2007-02";

        public static Message CreateTicketOrder(Guid eventId, string eventName, string location, string customer,
                                                string priceCategory) {
            return Message.CreateMessage(MessageVersion.Soap11,
                                         "http://reg.innoq.com/ns/wsdl/TicketService/2007-02/ITicketService/OrderTicket",
                                         new OrderWriter(eventId, eventName, location, customer, priceCategory));
        }
    }

    /// <summary>
    /// BodyWriter fr OrderTicket Request Message
    /// </summary>
    internal class OrderWriter : BodyWriter {
        private Guid id;
        private string eventName;
        private string location;
        private string customer;
        private string priceCategory;

        public OrderWriter(Guid pId, string pEventName, string pLocation, string pCustomer, string pPriceCategory)
            : base(false) {
            id = pId;
            eventName = pEventName;
            location = pLocation;
            customer = pCustomer;
            priceCategory = pPriceCategory;
        }

        ///<summary>
        ///When implemented, provides an extensibility point when the body contents are written.
        ///</summary>
        ///
        ///<param name="writer">The <see cref="T:System.Xml.XmlDictionaryWriter"></see> used to write out the message body.</param>
        protected override void OnWriteBodyContents(XmlDictionaryWriter writer) {
            writer.WriteStartElement("OrderTicket", ConsumerMessaging.TICKET_MESSAGE_XSD_NAMESPACE);
            writer.WriteStartElement("request", ConsumerMessaging.TICKET_MESSAGE_XSD_NAMESPACE);
            writer.WriteStartElement("event", ConsumerMessaging.TICKET_XSD_NAMESPACE);
            writer.WriteStartElement("id", ConsumerMessaging.TICKET_XSD_NAMESPACE);
            writer.WriteValue(id);
            writer.WriteEndElement(); // id
            writer.WriteStartElement("eventName", ConsumerMessaging.TICKET_XSD_NAMESPACE);
            writer.WriteValue(eventName);
            writer.WriteEndElement(); // eventName
            writer.WriteStartElement("location", ConsumerMessaging.TICKET_XSD_NAMESPACE);
            writer.WriteValue(location);
            writer.WriteEndElement(); // location
            writer.WriteEndElement(); // event
            writer.WriteStartElement("customer", ConsumerMessaging.TICKET_XSD_NAMESPACE);
            writer.WriteValue(customer);
            writer.WriteEndElement(); // customer
            writer.WriteStartElement("priceCategory", ConsumerMessaging.TICKET_XSD_NAMESPACE);
            writer.WriteValue(priceCategory);
            writer.WriteEndElement(); // priceCategory
            writer.WriteEndElement(); // request
            writer.WriteEndElement(); // OrderTicket
        }
    }

}