using System;
using System.IO;
using System.ServiceModel.Channels;
using System.Text;
using System.Xml;
using System.Xml.XPath;

namespace innoQ.TicketService {
    /// <summary>
    /// Hilfsmethoden fuer XML Messaging
    /// </summary>
    public static class TicketServiceMessaging {
        // Namespaces fr Messages und Dokumente
        public const string TICKET_XSD_NAMESPACE = "http://reg.innoQ.com/ns/xsd/TicketService/2007-02";
        public const string TICKET_MESSAGE_XSD_NAMESPACE = "http://reg.innoQ.com/ns/xsd/TicketService/Messages/2007-02";

        /// <summary>
        /// Erzeuge Response fr OrderTicket-Request.
        /// </summary>
        /// <param name="request"></param>
        /// <returns>Response</returns>
        public static Message CreateTicketOrderResponse(Message request) {
            // XPathNavigator (inkl. NamespaceManager) erzeugen, um auf den Request zuzugreifen
            XPathNavigator nav = new XPathDocument(request.GetReaderAtBodyContents()).CreateNavigator();
            XmlNamespaceManager nmMan = new XmlNamespaceManager(nav.NameTable);
            nmMan.AddNamespace("m", TICKET_MESSAGE_XSD_NAMESPACE);
            nmMan.AddNamespace("t", TICKET_XSD_NAMESPACE);

            // Request auswerten
            string customer = nav.SelectSingleNode("/m:OrderTicket/m:request/t:customer", nmMan).Value;
            string priceCategory = nav.SelectSingleNode("/m:OrderTicket/m:request/t:priceCategory", nmMan).Value;
            string ticketOrder = nav.SelectSingleNode("/m:OrderTicket/m:request", nmMan).InnerXml;
            decimal price = 10.0m;
            switch(priceCategory) {
                case "VIP" :
                    price *= 10.0m;
                    break;
                case "Standard" :
                    price *= 5.0m;
                    break;
            }

            // Erzeugen des Response
            return Message.CreateMessage(MessageVersion.Soap11,
                     "http://reg.innoq.com/ns/wsdl/TicketService/2007-02/ITicketService/OrderTicketResponse",
                     new TicketWriter(Guid.NewGuid(), ticketOrder, price));
        }
    }

    /// <summary>
    /// BodyWriter fr OrderTicket Response Message
    /// </summary>
    internal class TicketWriter : BodyWriter {
        private Guid id;
        private string ticketOrder;
        private decimal price;
	
        public TicketWriter(Guid pId, string pTicketOrder, decimal pPrice) : base(false) {
            id = pId;
            ticketOrder = pTicketOrder;
            price = pPrice;
        }

        ///<summary>
        ///When implemented, provides an extensibility point when the body contents are written.
        ///</summary>
        ///
        ///<param name="writer">The <see cref="T:System.Xml.XmlDictionaryWriter"></see> used to write out the message body.</param>
        protected override void OnWriteBodyContents(XmlDictionaryWriter writer) {
            writer.WriteStartElement("OrderTicketResponse", TicketServiceMessaging.TICKET_MESSAGE_XSD_NAMESPACE);
            writer.WriteStartElement("response", TicketServiceMessaging.TICKET_MESSAGE_XSD_NAMESPACE);
            writer.WriteStartElement("id", TicketServiceMessaging.TICKET_XSD_NAMESPACE);
            writer.WriteValue(id);
            writer.WriteEndElement(); // id
            writer.WriteStartElement("order", TicketServiceMessaging.TICKET_XSD_NAMESPACE);
            writer.WriteRaw(ticketOrder);
            writer.WriteEndElement(); // order
            writer.WriteStartElement("price", TicketServiceMessaging.TICKET_XSD_NAMESPACE);
            writer.WriteValue(price);
            writer.WriteEndElement(); // price
            writer.WriteEndElement(); // response
            writer.WriteEndElement(); // OrderTicketResponse
        }
    }
}