using System;
using System.Configuration;
using System.ServiceModel.Configuration;

namespace innoQ.WCF.Extensions.Validation {
    /// <summary>
    /// Service-Behavior Konfigurationselement fr das XsdValidationBehavior.
    /// </summary>
    public class XsdValidationBehaviorExtensionElement : BehaviorExtensionElement {
        private ConfigurationPropertyCollection properties;

        #region Properties
        public string XsdUri {
            get { return (string) base["xsdUri"]; }
            set { base["xsdUri"] = value; }
        }

        public string XsdTargetNamespace {
            get { return (string) base["xsdTargetNamespace"]; }
            set { base["xsdTargetNamespace"] = value; }
        }
        #endregion

        #region BehaviorExtensionElement Members
        ///<summary>
        ///Creates a behavior extension based on the current configuration settings.
        ///</summary>
        ///
        ///<returns>
        ///The behavior extension.
        ///</returns>
        ///
        protected override object CreateBehavior() {
            return new XsdValidationBehavior(XsdTargetNamespace, XsdUri);
        }

        ///<summary>
        ///Gets the type of behavior.
        ///</summary>
        ///
        ///<returns>
        ///A <see cref="T:System.Type"></see>.
        ///</returns>
        ///
        public override Type BehaviorType {
            get { return typeof (XsdValidationBehavior); }
        }

        ///<summary>
        ///Gets the collection of properties.
        ///</summary>
        ///
        ///<returns>
        ///The <see cref="T:System.Configuration.ConfigurationPropertyCollection"></see> of properties for the element.
        ///</returns>
        ///
        protected override ConfigurationPropertyCollection Properties {
            get {
                if (properties == null) {
                    properties = new ConfigurationPropertyCollection();
                    properties.Add(
                        new ConfigurationProperty("xsdUri", typeof (string), "", ConfigurationPropertyOptions.IsRequired));
                    properties.Add(
                        new ConfigurationProperty("xsdTargetNamespace", typeof (string), "",
                                                  ConfigurationPropertyOptions.IsRequired));
                }
                return properties;
            }
        }
        #endregion
    }
}